/*
 * file CqProvider.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.cq.CqProvider
 *
 *  Copyright IBM Corporation 2004, 2014.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp.cq;

import java.util.ArrayList;
import java.util.List;

import javax.wvcm.Feedback;
import javax.wvcm.ResourceList;
import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.StpLocation;
import com.ibm.rational.wvcm.stp.StpProvider;
import com.ibm.rational.wvcm.stp.cq.CqQuery.DisplayField;

/**
 * An extension of the StpProvider interface with additions specific to
 * ClearQuest databases.
 * <p>
 */
public interface CqProvider extends StpProvider
{
    /**
     * The name of a CqProvider class whose instances provide access only to
     * ClearQuest objects via a local installation of ClearQuest. Pass this name
     * to ProviderFactory.createProvider to obtain an object that implements
     * this interface.
     */
    String CQ_ONLY_PROVIDER_CLASS =
        "com.ibm.rational.stp.client.internal.cq.CqJniOnlyProvider";

    /**
     * A distinguished instance of List for use in the various CqContextResource
     * operations that take a delivery order parameter. It indicates that
     * resources modified by the operation are to be delivered in an unspecified
     * order. But the delivery is to be attempted only if the change context did
     * not contain the resource targeted by the operation. If the change context
     * already contained the targeted resource when the operation started, no
     * delivery is attempted. Thus AUTO behaves like {@link #DELIVER_ALL} or
     * {@link #HOLD} depending on the content of the change context at the start
     * of the operation.
     * <p>
     * Where an interface defines a second method that adds only a delivery
     * order List parameter to the first, the first method's behavior is defined
     * as if the second method had been passed this special instance. For
     * example, under this convention,
     * {@link CqContextResource#doWriteProperties(Feedback)} is defined as
     * 
     * <pre>
     * return doWriteProperties(feedback, CqProvider.AUTO);
     * </pre>
     */
    List<CqContextResource> AUTO = new ArrayList<CqContextResource>();

    /**
     * A distinguished instance of List for use in the various CqContextResource
     * operations that take a delivery order parameter. It indicates that all
     * modified resources in the change context are to be delivered in an
     * unspecified order.
     */
    List<CqContextResource> DELIVER_ALL = new ArrayList<CqContextResource>();

    /**
     * A distinguished instance of List for use in the various CqContextResource
     * operations that take a delivery order parameter. It indicates that
     * modified resources in the change context are not to be delivered, but are
     * to be held in the change context until the next explicit delivery
     * request.
     */
    List<CqContextResource> HOLD = new ArrayList<CqContextResource>();

    /**
     * A distinguished instance of List for use in the various CqContextResource
     * operations that take a delivery order parameter. It indicates that any
     * resource targeted by the current operation is to be delivered at the
     * successful termination of the operation.
     */
    List<CqContextResource> DELIVER = new ArrayList<CqContextResource>();

    /**
     * Returns a list of database sets accessible from this provider.
     * 
     * @param feedback The CqDbSet properties to be included on each returned
     *            CqDbSet proxy. If not <b>null</b>, an attempt will be made to
     *            log into each database set (using credentials provided by the
     *            provider's Callback for the database set). If login to a
     *            database set is not successful, the resource error field of
     *            the corresponding CqDbSet proxy will be non-null and contain
     *            the exception thrown by the provider's Callback object.
     *            <p>
     *            To construct a list of accessible user databases, request the
     *            {@link CqDbSet#ACCESSIBLE_DATABASES} property from each
     *            database set.
     * @return A list of CqDbSet proxies for the potentially accessible database
     *         sets.
     * @throws WvcmException If an error other than a failed login are detected.
     *             If login to a master database is required (because properties
     *             have been requested) and the login fails, a proxy for the
     *             database set is included in the result list, but the value of
     *             CqDbSet.getResourceError() will be non-null and that proxy
     *             will contain none of the requested properties.
     */
    ResourceList<CqDbSet> doGetDbSetList(Feedback feedback)
        throws WvcmException;

    /**
     * Creates a proxy for a ClearQuest action resource.
     * 
     * @param location StpLocation for a ClearQuest action.
     * 
     * @return The new CqAction proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqAction cqAction(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest attachment resource.
     * 
     * @param location StpLocation for a ClearQuest attachment.
     * 
     * @return The new CqAttachment proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqAttachment cqAttachment(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest attachment folder resource.
     * 
     * @param location StpLocation for a ClearQuest attachment folder.
     * 
     * @return The new CqAttachmentFolder proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqAttachmentFolder cqAttachmentFolder(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest database set resource.
     * 
     * @param location StpLocation for a ClearQuest db-set.
     * 
     * @return The new CqDbSet proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqDbSet cqDbSet(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest dynamic choice list resource.
     * 
     * @param location StpLocation for a ClearQuest dynamic choice list.
     * 
     * @return The new CqDynamicChoiceList proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqDynamicChoiceList cqDynamicChoiceList(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest field definition resource.
     * 
     * @param location StpLocation for a ClearQuest field definition.
     * 
     * @return The new CqFieldDefinition proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqFieldDefinition cqFieldDefinition(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest group resource.
     * 
     * @param location StpLocation for a ClearQuest group.
     * 
     * @return The new CgGroup proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqGroup cqGroup(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest hook resource.
     * 
     * @param location StpLocation for a ClearQuest hook.
     * 
     * @return The new CgHook proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqHook cqHook(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest query resource.
     * 
     * @param location StpLocation for a ClearQuest query.
     * 
     * @return The new Query proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqQuery cqQuery(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest query folder resource.
     * 
     * @param location StpLocation for a ClearQuest query folder.
     * 
     * @return The new CqQueryFolder proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqQueryFolder cqQueryFolder(StpLocation location);

    /**
     * Creates a proxy for a resource in a ClearQuest query folder.
     * 
     * @param location StpLocation for a ClearQuest query folder item.
     * 
     * @return The new CqQueryFolderItem proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqQueryFolderItem cqQueryFolderItem(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest record resource.
     * 
     * @param location StpLocation for a ClearQuest record.
     * 
     * @return The new CqRecord proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqRecord cqRecord(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest record-type resource.
     * 
     * @param location StpLocation for a ClearQuest record-type.
     * 
     * @return The new CqRecordType proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqRecordType cqRecordType(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest replica resource.
     * 
     * @param location StpLocation for a ClearQuest replica.
     * 
     * @return The new CqReplica proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqReplica cqReplica(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest report resource.
     * 
     * @param location StpLocation for a ClearQuest report.
     * 
     * @return The new CqReport proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqReport cqReport(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest report format resource.
     * 
     * @param location StpLocation for a ClearQuest report format.
     * 
     * @return The new CqReportFormat proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqReportFormat cqReportFormat(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest user resource.
     * 
     * @param location StpLocation for a ClearQuest user.
     * 
     * @return The new CqUser proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqUser cqUser(StpLocation location);

    /**
     * Creates a proxy for a ClearQuest user database resource.
     * 
     * @param location StpLocation for a ClearQuest user database.
     * 
     * @return The new CqUserDb proxy.
     * 
     * @throws WvcmException if StpLocation is not a ClearQuest resource
     *             location.
     */
    CqUserDb cqUserDb(StpLocation location);

    /**
     * Create a new CqFieldValue structure for a field of a given name and type.
     * 
     * @param name The PropertyName for the field.
     * @param type The type of the field.
     * @return A CqFieldValue structure.
     */
    <U> CqFieldValue<U> cqFieldValue(PropertyName<U> name,
                                     CqFieldValue.ValueType type);

    /**
     * An extension of the
     * {@link com.ibm.rational.wvcm.stp.StpProvider.StpProductInfo} interface
     * specifying the additional information available from a ClearQuest
     * repository.
     */
    static public interface CqProductInfo extends StpProductInfo
    {
        /**
         * @return The name of the default database set on the server
         */
        String getDefaultDbSetName();

        /**
         * @return The major version number of the object model supported by
         * the server.
         */
        long getObjectModelVersionMajor();

        /**
         * @return The minor version number of the object model supported by
         * the server.
         */
        long getObjectModelVersionMinor();
    }

    /**
     * Constructs a visible DisplayField object for defining Query display
     * fields
     * 
     * @param path A CqFieldDefinition[] specifying the field path for the
     *            display field. The first entry of the array must be a
     *            CqFieldDefinition for a field of the query's primary record
     *            type. Entry N is allowed only if entry N-1 is a field of type
     *            {@link CqFieldValue.ValueType#RESOURCE} or
     *            {@link CqFieldValue.ValueType#RESOURCE_LIST} and entry N is a
     *            field of the record type named by the
     *            {@link CqFieldDefinition#REFERENCED_RECORD_TYPE}
     * 
     * @return A DisplayField object for specifying a column of the result set
     *         of a query.
     */
    public DisplayField buildDisplayField(CqFieldDefinition... path);

    /**
     * Constructs a new DisplayField object for this Query proxy
     * 
     * @param path The field path for the display field (see
     *            {@link #buildDisplayField(CqFieldDefinition[])})
     * @param isVisible Whether or not the display field is to be visible
     * 
     * @return A DisplayField object for specifying a column of the result set
     *         of a query initialized to the given path a visibility.
     */
    public DisplayField buildDisplayField(CqFieldDefinition[] path,
                                          boolean isVisible);

    /**
     * Constructs a new {@link CqQuery.FilterLeaf} object to be used in the
     * creation or modification of a query's filtering expression.
     * 
     * @param operation The comparison Operation code for this filter node. Must
     *            not be <b>null</b>.
     * @param source The field path representing the left operand of the
     *            operation. See {@link CqQuery.FilterLeaf#getSource}
     *            documentation for more discussion of field paths. Must not be
     *            <b>null</b>.
     * @param targets An array of righthand side target types/values suitable as
     *            an argument to {@link CqQuery.FilterLeaf#setTargets(Object[])}
     * 
     * @return A CqQuery.FilterLeaf object specifying the comparison of a field
     *         value against a target value
     */
    public CqQuery.FilterLeaf buildFilterLeaf(CqFieldDefinition[] source,
                                              CqQuery.Filter.Operation operation,
                                              Object... targets);

    /**
     * Constructs a new CqQuery.FilterNode object for the purpose of creating or
     * modifying the filtering expression of a query.
     * 
     * @param operation Either {@link CqQuery.Filter.Operation#CONJUNCTION} or
     *            {@link CqQuery.Filter.Operation#DISJUNCTION}. Must not be
     *            <b>null</b>.
     * @param operands The filter subexpressions to be combined to form a new
     *            filter expression. Must not be <b>null</b>.
     * 
     * @return A CqQuery.FilterNode object for specifying the logical
     *         combination of filtering subexpressions
     */
    public CqQuery.FilterNode buildFilterNode(CqQuery.Filter.Operation operation,
                                              CqQuery.Filter... operands);

    /**
     * Constructs a new CqQuery.FilterNode object for the purpose of creating or
     * modifying the filtering expression of a query.
     * 
     * @param operation Either {@link CqQuery.Filter.Operation#CONJUNCTION} or
     *            {@link CqQuery.Filter.Operation#DISJUNCTION}. Must not be
     *            <b>null</b>.
     * @param operands The filter subexpressions to be combined to form a new
     *            filter expression. Must not be <b>null</b>.
     * 
     * @return A CqQuery.FilterNode object for specifying the logical
     *         combination of filtering subexpressions
     */
    public CqQuery.FilterNode buildFilterNode(CqQuery.Filter.Operation operation,
                                              CqQuery.FilterLeaf... operands);

    /**
     * Constructs a new CqQuery.FilterNode object for the purpose of creating or
     * modifying the filtering expression of a query.
     * 
     * @param operation Either {@link CqQuery.Filter.Operation#CONJUNCTION} or
     *            {@link CqQuery.Filter.Operation#DISJUNCTION}. Must not be
     *            <b>null</b>.
     * @param operands The filter subexpressions to be combined to form a new
     *            filter expression. Must not be <b>null</b>.
     * 
     * @return A CqQuery.FilterNode object for specifying the logical
     *         combination of filtering subexpressions
     */
    public CqQuery.FilterNode buildFilterNode(CqQuery.Filter.Operation operation,
                                              CqQuery.FilterNode... operands);

    /*
     * Keep track of remote address of client/browser making the requests.
     */
    
    /**
     * Get the remote address that is making this CQ request.
     * @return a string representing the remote address.
     */
    public String getRemoteAddress();
    
    /**
     * Set the remoteAddress for this provider. This is the address
     * that is making the current request.
     * @param remoteAddress A string representing the address that is
     * 				making this request.
     */
    public void setRemoteAddress(String remoteAddress);

}
